/*
Copyright (C) 2009 -  Claudio Salvadori 
This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#ifndef CTARGET_H
#define CTARGET_H

#include <FL/Fl.H>
#include <FL/Fl_Double_Window.H>
#include <FL/Fl_Box.H>
#include <Fl/Fl_Image.H>
#include <stdio.h>
#include <string>

#include "IRenderer.h"
#include "CXmlParser.h"

 /**
 *\file CTarget.h
 *
 *\date  Created on: 31-oct-2009
 *\author  Author: Claudio Salvadori
 *\brief  Class to render a black screen to show the position
 * 	of a target.
 */
 
 /**
 *\class CTarget
 *\brief  Implementation of IRenderer interface to render a black screen to 
  * 	show the position of a target using FLTK libraries
 */

class CTarget: public IRenderer{
public:
	~CTarget();
	
	/** 	\brief This function, used by the object factory, return an istance
	*		of CTarget class
	*	\param node	pointer to a generic node (in this case DOMNode) where
	*			extracts the Target presets.
	*/
	static IRenderer *createObj(const void* node);
	
	/** 	\brief Function to set the data type
	* 	\param vType	data type*/
	void setType(var_type vType);

	/** 	\brief Function to render the Target
	* 	\param data 	pointer to 2D target vector*/
	void render(void* data, unsigned long size);

private:
				
	/**	\brief Constructor: it intializes the windows and the screen (FLTK libraries)
	* 	\param node	pointer to parent DOM node*/
	CTarget(const DOMNode* node);
	
	/**	\brief This function parse the xml-file to extract the target presets
	* 	\param node	pointer to parent DOM node*/
	void targetPreset(const DOMNode* node);
	
	DOMNode* 		tnode_;		/**<target presets DOM node*/
	
	Fl_Double_Window*	screen_wnd;	/**<Pointer to the window*/
	Fl_Box*			screen_img;	/**<Pointer to the screen*/
	
	unsigned char*		image_; 	/**<All 0 vector (black image)*/
	
	var_type 		vType_;		/**<Target variable type*/
	
	std::string		name_;		/**<Window caption*/
	unsigned int 		w_;		/**<Image width*/
	unsigned int 		h_;		/**<Image height*/
	
	CXMLParser*		 parser_;	/**<Pointer to Xml parser*/
};
#endif
