/*
Copyright (C) 2009 -  Claudio Salvadori 
This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

/**
 *\file CDaq.cpp
 *\brief  CDaq methods development
 */

#include "CDaq.h"

#define XML_PREFIX			0		/**< Value to be added to CXmlParser return value
										to be translated to CDaq return value \see daq_ret \see x_ret*/
#define READER_PREFIX		100		/**< Value to be added to CReader return value
										to be translated to CDaq return value \see daq_ret \see s_ret*/
#define FORMATTER_PREFIX	200		/**< Value to be added to CFormatter return value
										to be translated to CDaq return value \see daq_ret \see f_ret*/

CDaq::CDaq(char *xmlFile){
	//Blocks init
	daq_Xml = NULL;	
	daq_Reader = NULL;
	daq_Formatter = NULL;
	
	//Data structure init
	daq_Definition = NULL;
	daq_Definition = new SDef;
	daq_Metadata = NULL;
	daq_Metadata = new vector<struct SMData>;
	daq_Message = NULL;
	daq_Message = new vector<struct SMessage>;
	daq_Data = NULL;
	daq_Data = new vector<SData>;
	
	//Creation CXml class
	daq_Xml = new CXml(xmlFile, daq_Definition, daq_Metadata, &daq_title);
	daq_Reader = new CReader();
}
		
CDaq::~CDaq(){
	//Blocks delete 
	delete daq_Xml;	
	delete daq_Reader;
	delete daq_Formatter;
	
	//Data structure delete
	delete daq_Definition;
	delete daq_Metadata;
	delete daq_Message;
	delete daq_Data;
}

daq_ret CDaq::init(void){
	x_ret r_xml;
	s_ret r_reader;
	f_ret r_formatter;
	
	r_xml = daq_Xml->init();
	
	#ifdef DEBUG
	printf("IXml::Init return: %d \n", r_xml);
	#endif
	
	if(r_xml != X_RET_OK)
		return (daq_ret)(XML_PREFIX + r_xml);
	
	r_xml = daq_Xml->parseXml();
	#ifdef DEBUG
	printf("IXml::Parse return: %d \n", r_xml);
	#endif
	if(r_xml != X_RET_OK)
		return (daq_ret)(XML_PREFIX + r_xml);
		
	r_reader = daq_Reader->getDefinition(*daq_Definition);
	#ifdef DEBUG
	printf("IStreamReader::getDefinition return: %d \n", r_reader);
	#endif
	if(r_reader != S_RET_OK)
		return (daq_ret)(READER_PREFIX + r_reader);
		
	daq_Formatter = new CFormatter(daq_Data);
	
	r_formatter = daq_Formatter->init(daq_Metadata);
	#ifdef DEBUG
	printf("IFormatter::init return: %d \n", r_reader);
	#endif
	if(r_formatter != F_RET_OK)
		return (daq_ret)(FORMATTER_PREFIX + r_formatter);
		
	return D_RET_OK;
	
}
	
string CDaq::getTitle(void){
	return daq_title;
}

daq_ret CDaq::getData(vector< vector<SData> >* data){
	s_ret r_reader;
	f_ret r_formatter;
	
	//Reception, decoding, division in messages from steam source
	r_reader = daq_Reader->retStream((vector<struct SMessage>*)daq_Message);
	//#ifdef DEBUG
	printf("IStreamReader::retStream return: %d \n", r_reader);
	//#endif
	if(r_reader != S_RET_OK)
		return (daq_ret)(READER_PREFIX + r_reader);
	
	//buffer clear
	data->clear();
	
	for(unsigned int i = 0; i < daq_Message->size(); i++){
		r_formatter = daq_Formatter->getData((*daq_Message)[i].me_Data);
		#ifdef DEBUG
		printf("CFormatter::getData return: %d \n", r_formatter);
		#endif
		if(r_formatter != F_RET_OK)
			return (daq_ret)(FORMATTER_PREFIX + r_formatter);
		
		data->push_back(*daq_Data);
	}
	
	return D_RET_OK;
}
